@@ -30,10 +30,11 @@ const (
 )
 
 var (
-	keys         = []string{}
-	keysExpiring = []string{}
-	listKeys     = []string{}
-	ts           = int32(time.Now().Unix())
+	keys            = []string{}
+	keysExpiring    = []string{}
+	listKeys        = []string{}
+	singleStringKey string
+	ts              = int32(time.Now().Unix())
 
 	dbNumStr        = "11"
 	altDBNumStr     = "12"
@@ -64,6 +65,11 @@ func getTestExporterWithOptions(opt Options) *Exporter {
 	return e
 }
 
+func getTestExporterWithAddr(addr string) *Exporter {
+	e, _ := NewRedisExporter(addr, Options{Namespace: "test", Registry: prometheus.NewRegistry()})
+	return e
+}
+
 func setupKeys(t *testing.T, c redis.Conn, dbNumStr string) error {
 	if _, err := c.Do("SELECT", dbNumStr); err != nil {
 		log.Printf("setupDBKeys() - couldn't setup redis, err: %s ", err)
@@ -97,6 +103,8 @@ func setupKeys(t *testing.T, c redis.Conn, dbNumStr string) error {
 	c.Do("SADD", TestSetName, "test-val-1")
 	c.Do("SADD", TestSetName, "test-val-2")
 
+	c.Do("SET", singleStringKey, "this-is-a-string")
+
 	// Create test streams
 	c.Do("XGROUP", "CREATE", TestStreamName, "test_group_1", "$", "MKSTREAM")
 	c.Do("XGROUP", "CREATE", TestStreamName, "test_group_2", "$", "MKSTREAM")
@@ -130,6 +138,7 @@ func deleteKeys(c redis.Conn, dbNumStr string) {
 
 	c.Do("DEL", TestSetName)
 	c.Do("DEL", TestStreamName)
+	c.Do("DEL", singleStringKey)
 }
 
 func setupDBKeys(t *testing.T, uri string) error {
@@ -216,6 +225,26 @@ func TestIncludeSystemMemoryMetric(t *testing.T) {
 	}
 }
 
+func TestIncludeConfigMetrics(t *testing.T) {
+	for _, inc := range []bool{false, true} {
+		r := prometheus.NewRegistry()
+		ts := httptest.NewServer(promhttp.HandlerFor(r, promhttp.HandlerOpts{}))
+		e, _ := NewRedisExporter(os.Getenv("TEST_REDIS_URI"), Options{Namespace: "test", InclConfigMetrics: inc})
+		r.Register(e)
+
+		what := `test_config_key_value{key="appendonly",value="no"}`
+
+		body := downloadURL(t, ts.URL+"/metrics")
+		if inc && !strings.Contains(body, what) {
+			t.Errorf("want metrics to include test_config_key_value, have:\n%s", body)
+		} else if !inc && strings.Contains(body, what) {
+			t.Errorf("did NOT want metrics to include test_config_key_value, have:\n%s", body)
+		}
+
+		ts.Close()
+	}
+}
+
 func TestNonExistingHost(t *testing.T) {
 	e, _ := NewRedisExporter("unix:///tmp/doesnt.exist", Options{Namespace: "test"})
 
@@ -340,14 +369,14 @@ func init() {
 	}
 
 	for _, n := range []string{"john", "paul", "ringo", "george"} {
-		key := fmt.Sprintf("key_%s_%d", n, ts)
-		keys = append(keys, key)
+		keys = append(keys, fmt.Sprintf("key_%s_%d", n, ts))
 	}
 
+	singleStringKey = fmt.Sprintf("key_string_%d", ts)
+
 	listKeys = append(listKeys, "beatles_list")
 
 	for _, n := range []string{"A.J.", "Howie", "Nick", "Kevin", "Brian"} {
-		key := fmt.Sprintf("key_exp_%s_%d", n, ts)
-		keysExpiring = append(keysExpiring, key)
+		keysExpiring = append(keysExpiring, fmt.Sprintf("key_exp_%s_%d", n, ts))
 	}
 }
